/****************************************************************************
** Implementation of class QDVDAuthorInit
**
** Created: Wed Jun 02 07:53:05 2004
**      by: Varol Okan using the kate editor
**
** This class initialises the QDVDAuthor application with the
** last used settings.
**
** It is mainly used, to handle the CommandQueueDialogs options
** which can now be changed by the user.
**
****************************************************************************/

#include <qmessagebox.h>

#include "execute.h"
#include "qdvdauthorinit.h"
#include "sourcefileentry.h"
#include "dragndropcontainer.h"
#include "qplayer/mediainfo.h"
#include "qplayer/mediacreator.h"

#define MAX_KEYWORDS 12

QDVDAuthorInit::MenuBlock::MenuBlock ()
{ 
	bValid = false;  
	bHidden = false;
	blockMovieBackground.qsBlockTag = QString ( BLOCK_MOVIE_BACKGROUND );
	blockImageBackground.qsBlockTag = QString ( BLOCK_IMAGE_BACKGROUND );
	blockEmptySound.qsBlockTag = QString ( BLOCK_EMPTY_SOUND );
	blockMplex.qsBlockTag  = QString ( BLOCK_MPLEX );
	blockSpumux.qsBlockTag = QString ( BLOCK_SPUMUX );
	blockOthers.qsBlockTag = QString ( BLOCK_OTHERS );
};

// FIXME:
// As of June 8'th 2004 this class is done 90%.
// Done ->
// o Loading of init file.
// o Saving the MenuBlock stored data back to the ini - file.
// o Saving of default init file.
// o The function to store an init file depending on the users modifications.
// o Added buttons to the CommandQueueDialog (DialogExecute)
//
// Missing ->
// o Some more sanity checks.
// o The function convertSoundFile ()
//////////////////////////////////////////////////////////////////////////////////////////////
/* NEW STUFF !!!
<ExecuteInterface><Burning blockname="xyz"><Block>
  <SubBlock number=[1 .. N] active=[true/false]>
-- Only if active is specified put check box in front ...
   When check box is unset, gray out the command.
-- If command == "-- Rendder Menu --" then grey out drop down box.
*/ 
QDVDAuthorInit::QDVDAuthorInit()
{
	initMe();
	readIniFile();
}

QDVDAuthorInit::QDVDAuthorInit(DragNDropContainer *pDNDContainer)
{
	initMe();
	m_pDragNDropContainer = pDNDContainer;
	readIniFile();
}

QDVDAuthorInit::QDVDAuthorInit(QValueList<Utils::toolsPaths *> &listPaths, DragNDropContainer *pDNDContainer)
{
	// First we init some data
	initMe();
	// Then we load the current ini - file and set all Internals to those values
	readIniFile();
	// And here we set the dragNDropContainer AFTER laoding to keep the values preserved !
	m_pDragNDropContainer = pDNDContainer;
	// Here we indicate that we do not want to delete the objects when deleting this object ...
	m_bProtectToolsObjects = true;
	// and then we set the paths list to the one we want it to be ...
	setToolsPaths (listPaths);
}

QDVDAuthorInit::~QDVDAuthorInit()
{
	int t;
	// -2 because the last two entries are static ones ...
	for (t=0;t<(int)m_listExecute.count()-2;t++)
		delete m_listExecute[t];
	for (t=0;t<(int)m_arrayColors.count();t++)
		delete m_arrayColors[t];
	if (m_bOwnerOfHistoryPix)	{
		// If this flag is not set means that some other object (QDVDAuthor e.g.) is the master of these.
		for (t=0;t<(int)m_listHistoryPix.count();t++)
			delete m_listHistoryPix[t];
	}
	// Note: Since we clear the list every time getToolsPaths list is called we can delete 
	//       All objects which are at this time still in the list. 
	if (!m_bProtectToolsObjects)	{
		for (t=0;t<(int)m_listToolsPaths.count()-2;t++)
			delete m_listToolsPaths[t];
	}
}

bool QDVDAuthorInit::initMe()
{
	int t;
	m_iMaxHistoryFiles     = 10;
	m_bOwnerOfHistoryPix   = true;
	m_iAutosave            = 300;
 	m_iColorCounter        = 0;
	m_iCurrentBlockID      = 0;	// To uniquely identify each block of commands ...
	m_bProtectToolsObjects = false;
	m_bVisibleRegion       = false;

	m_qsTempPath           = Global::qsTempPath + QString ("/") + Global::qsProjectName;
	m_qsProjectPath        = Global::qsProjectPath;
	m_qsDvdauthorFile      = m_qsTempPath + QString ("/") + QString(DVDAUTHOR_XML);
	m_pDragNDropContainer  = NULL;

	m_convertMenuBlock.qsMenuTag   = QString (TAG_CONVERT);
	m_palMenuBlock.qsMenuTag       = QString (TAG_PAL);
	m_ntscMenuBlock.qsMenuTag      = QString (TAG_NTSC);
	m_dvdauthorMenuBlock.qsMenuTag = QString (TAG_DVDAUTHOR);
	m_burningMenuBlock.qsMenuTag   = QString (TAG_BURNING);
	if (m_listExecute.count() > 2)	{
		for (t=0;t<(int)m_listExecute.count()-2;t++)
			delete m_listExecute[t];
	}
	m_listExecute.clear ();
	return true;
}

QString QDVDAuthorInit::preferredEngine()
{
	return m_qsPreferredEngine;
}

bool QDVDAuthorInit::visibleRegion()
{
	return m_bVisibleRegion;
}

void QDVDAuthorInit::setToolsPaths (QValueList<Utils::toolsPaths *> &listPaths)
{
	// Note: Deletion of the objects in the list only because we know where this function is called from.
	// I.e. every time QDVDAuthorInit is called it'll create a new list and new objects. Thus when setting 
	//      the list manually then we have to get rid of the created Objects.
	uint t;
	if (m_listToolsPaths.count() > 0)	{
		for (t=0;t<m_listToolsPaths.count();t++)
			delete m_listToolsPaths[t];
		m_listToolsPaths.clear();
	}
	m_listToolsPaths = listPaths;
}

QValueList<Utils::toolsPaths *> &QDVDAuthorInit::getToolsPaths ()
{
//	return m_listToolsPaths;

	static QValueList<Utils::toolsPaths *> listNew;
	listNew = m_listToolsPaths;
	// Note: Here we return the list and reset the member list. 
	// We do this because we want to ensure the objects in this list are
	// created/deleted only once.
	m_listToolsPaths.clear();
	return listNew;
}

void QDVDAuthorInit::mergeToolsPaths (QValueList<Utils::toolsPaths *> listToolsPaths)
{
	// This function will merge the information found in the init file with the information in the 
	// static structure found in global.h
	uint t, i;
	QValueList<Utils::toolsPaths *> newToolsPathsList;
	Utils::toolsPaths *pNewEntry;
	// Okay first we create a fresh and clean list form the static list defined in global.h
	struct structTools {
		char *pExecutableName;
		char *pDisplayName;
		char *pDescription;
	};
	const structTools toolsArray[] = { EXTERNAL_TOOLS };
	uint iNrOfTools = sizeof (toolsArray) / ((sizeof (char *) * 3));
	for (t=0;t<iNrOfTools;t++)	{
		pNewEntry = new Utils::toolsPaths;
		pNewEntry->qsExecutableName = QString (toolsArray[t].pExecutableName);
		newToolsPathsList.append (pNewEntry);
	}
	// then we copy all values over from the current list
	for (t=0;t<newToolsPathsList.count();t++)	{
		for (i=0;i<m_listToolsPaths.count();i++)	{
			if (m_listToolsPaths[i]->qsExecutableName == newToolsPathsList[t]->qsExecutableName)	{
				newToolsPathsList[t]->qsFullPath = m_listToolsPaths[i]->qsFullPath;
				newToolsPathsList[t]->bPresent = m_listToolsPaths[i]->bPresent;
				i = m_listToolsPaths.count();
			}
		}
	}
	// finally we copy all values over found in the ini file list ...
	for (t=0;t<listToolsPaths.count();t++)	{
		for (i=0;i<newToolsPathsList.count();i++)	{
			if (newToolsPathsList[i]->qsExecutableName == listToolsPaths[t]->qsExecutableName)	{
				newToolsPathsList[i]->qsFullPath = listToolsPaths[t]->qsFullPath;
				i = newToolsPathsList.count();
			}
		}
	}
	// And at the end we check if the executable exists in the place specified ...
	QString qsPath;
	QFileInfo fileInfo;
	for (t=0;t<newToolsPathsList.count();t++)	{
		fileInfo.setFile(newToolsPathsList[t]->qsFullPath);
		if (fileInfo.exists())
			newToolsPathsList[t]->bPresent = true;
		else
			newToolsPathsList[t]->bPresent = false;
	}
	// Okay now we can delete the entries in the current list ...
	for (t=0;t<m_listToolsPaths.count();t++)
		delete m_listToolsPaths[t];
	m_listToolsPaths.clear ();
	// now copy the new list over and make it the current list
	m_listToolsPaths = newToolsPathsList;
	// Done here, lets go home ...
}

QStringList QDVDAuthorInit::getHistory ()
{
	return m_listHistory;
}

QValueList<QPixmap *> QDVDAuthorInit::getHistoryPix ()
{
	// This flag indicates that we moved all Pixmaps to the caller.
	m_bOwnerOfHistoryPix = false;
	return m_listHistoryPix;
}

int QDVDAuthorInit::maxHistoryFiles ()
{
	return m_iMaxHistoryFiles;
}

int QDVDAuthorInit::getAutosave ()
{
	return m_iAutosave;
}

void QDVDAuthorInit::setAutosave (int iAutosave)
{
	m_iAutosave = iAutosave;
}

void QDVDAuthorInit::setVisibleRegion (bool bVisibleRegion)
{
	m_bVisibleRegion = bVisibleRegion;
}

void QDVDAuthorInit::setHistory (int iMaxHistoryFiles, QStringList listHistory, QValueList<QPixmap *> listHistoryPix)
{
	m_iMaxHistoryFiles   = iMaxHistoryFiles;
	m_listHistory        = listHistory;
	m_listHistoryPix     = listHistoryPix;
	m_bOwnerOfHistoryPix = false;
}

bool QDVDAuthorInit::readIniFile ()
{
	// This function reads in the xml - init - file for 'Q' DVD-Author and
	// generates the neccesary data structures to be used by appendMenu.
	float fStoredVersion;
	int iReturn;
	// Assign the file
	QString qsInitFile = QDir::homeDirPath();
	qsInitFile.append(INIT_DIR_NAME);
	QDir iniDir (qsInitFile);
	if (!iniDir.exists())
		iniDir.mkdir (qsInitFile);

	qsInitFile.append(INIT_FILE_NAME);
	QFile initFile(qsInitFile);
	if (!initFile.open(IO_ReadWrite))
		return false;

	QDomDocument xmlDoc( INIT_DOCTYPE );
	if (!xmlDoc.setContent (&initFile))	{
		// Error handling ...
		initFile.close();
		iReturn = QMessageBox::warning ( NULL, QObject::tr ("QDVDAuthor - init file seems to be defective."),
			QObject::tr ("The file %1%2 seems  to have a problem. Do you want to reset this file ?").arg(INIT_DIR_NAME).arg(INIT_FILE_NAME),
			QMessageBox::Yes, QMessageBox::No);
		if (iReturn == QMessageBox::Yes)	{
			resetInitFile ();
			return initMe ();
		}
	}
	// And at last lets try to read the information of the file.
	QDomElement docElem = xmlDoc.documentElement();
	// Sanity check ...
	if (docElem.tagName() != QString (TAG_INIT))
		return false;

	QDomAttr a = docElem.attributeNode ( ATTRIB_VERSION );
	fStoredVersion = a.value().toFloat();
	a = docElem.attributeNode ( ATTRIB_VISIBLE_REGION );
	m_bVisibleRegion = false;
	if (a.value() == QString ("true"))
		m_bVisibleRegion = true;
	a = docElem.attributeNode ( ATTRIB_MAX_HISTORY );
	if (a.value().toInt() > 0)
		m_iMaxHistoryFiles = a.value().toInt();
	a = docElem.attributeNode ( ATTRIB_AUTOSAVE );
	if (a.value().toInt() > 0)
		m_iAutosave = a.value().toInt();
	if (fStoredVersion > (float)INIT_VERSION)	{
		iReturn = QMessageBox::warning ( NULL, QObject::tr ("QDVDAuthor - init file is newer than the current version of 'Q' DVD-Author."),
			QObject::tr ("The file %1%2 was created with a more recent version.\nThis could lead to problems.\nDo you want to reset this file ?").arg(INIT_DIR_NAME).arg(INIT_FILE_NAME),
			QMessageBox::Yes, QMessageBox::No);
		if (iReturn == QMessageBox::Yes)	{
			resetInitFile ();
			return initMe ();
		}
	}
	else if (fStoredVersion < (float)INIT_VERSION)	{
		// Here we tell the user and then reset the initfile.
		iReturn = QMessageBox::warning ( NULL, QObject::tr ("QDVDAuthor - init file is outdated."),
			QObject::tr ("The file %1%2 was created with an older version of 'Q' DVD-Author.\nThis could lead to problems.\n\nDo you want to reset this file ?").arg(INIT_DIR_NAME).arg(INIT_FILE_NAME),
			QMessageBox::Yes, QMessageBox::No);
		if (iReturn == QMessageBox::Yes)	{
			resetInitFile ();
			return initMe ();
		}
	}

	// Here we get the next step in the tree down ... ExecuteInterface
	uint t;
	QStringList listCommands;
	QValueList<QString> listColors;
	QValueList<int>     listOrder;
	QDomNode xmlNode;
	QDomElement childElem;
	QDomNode xmlExecuteInterfaceNode = docElem.firstChild();
	bool bHidden;
	MenuBlock *pMenuBlock;
	QValueList<Utils::toolsPaths *> listToolsPaths;
	m_listHistory.clear ();
	m_iCurrentBlockID = 0;
	// Okay
	listCommands.clear();
	while (!xmlExecuteInterfaceNode.isNull())	{
		bHidden = false;
		childElem = xmlExecuteInterfaceNode.toElement();
		if (childElem.tagName() == QString(TAG_EXECUTE))	{
			// Correctly detects a ExecuteInterface - tag
			// So lets get the only attribute for ExecuteInterface
			a = childElem.attributeNode ( ATTRIB_HIDDEN );
			if (a.value() == QString ("true"))
				bHidden = true;
			// Here we get the next step in the tree down ... Pal, or Ntsc or Dvdauthor or Burning
			xmlNode = childElem.firstChild();
			while (!xmlNode.isNull())	{
				childElem = xmlNode.toElement();
				pMenuBlock = NULL;
				if (childElem.tagName() == QString (TAG_CONVERT))
					pMenuBlock = &m_convertMenuBlock;
				else if (childElem.tagName() == QString (TAG_PAL))
					pMenuBlock = &m_palMenuBlock;
				else if (childElem.tagName() == QString (TAG_NTSC))
					pMenuBlock = &m_ntscMenuBlock;
				else if (childElem.tagName() == QString (TAG_DVDAUTHOR))
					pMenuBlock = &m_dvdauthorMenuBlock;
				else if (childElem.tagName() == QString (TAG_BURNING))
					pMenuBlock = &m_burningMenuBlock;
				if (!pMenuBlock)
					continue;
				if (!readMenuBlock (pMenuBlock, &childElem))
					return false;
				pMenuBlock->bHidden = bHidden;
				xmlNode = xmlNode.nextSibling();
			}
		}
		else if (childElem.tagName() == QString(TAG_PREFERRED_ENGINE))
			m_qsPreferredEngine = childElem.text();
		else if (childElem.tagName() == QString(TAG_COLOR))	{
			QDomAttr a = childElem.attributeNode ( ATTRIB_NUMBER );
			listOrder.append(a.value().toInt());
			listColors.append(childElem.text());
			m_arrayColors.append(new QColor());
		}
		else if (childElem.tagName() == QString(TAG_PATH))	{
			QDomAttr a = childElem.attributeNode ( ATTRIB_NAME );
			Utils::toolsPaths *pPath = new Utils::toolsPaths;
			pPath->qsFullPath = childElem.text();
			pPath->qsExecutableName = a.value ();
			listToolsPaths.append(pPath);
		}
		else if (childElem.tagName() == QString(TAG_HISTORY))	{
			QPixmap *pPixmap = new QPixmap;
			Utils theUtil;
			QDomAttr a = childElem.attributeNode ( ATTRIB_NAME );
			QString qsImage = childElem.text();
			pPixmap->load (qsImage);
			m_listHistory.append(a.value());
			m_listHistoryPix.append(pPixmap);
/* The original version when we stored the images in the ini - file.
			QPixmap *pPixmap = new QPixmap;
			Utils theUtil;
			QDomAttr a = childElem.attributeNode ( ATTRIB_NAME );
			QString qsImage = childElem.text();
			m_listHistory.append(a.value());
			m_listHistoryPix.append(pPixmap);
			// And now we convert the string to a QImage - object
			QImage theImage = theUtil.convertStringToImage (qsImage);
			pPixmap->convertFromImage(theImage);
*/
		}
		else if (childElem.tagName() == QString(TAG_DRAGNDROP))	{
			if (m_pDragNDropContainer)	{
				DragNDropContainer *p = m_pDragNDropContainer;
				int iSizeWidth, iSizeHeight;
				// Here we add the DragNDrop - stuff to the init - file ...
				QDomAttr a = childElem.attributeNode ( ATTRIB_DND_THUMBNAIL );
				// The size is something like 50x50 so we ought to split
				int iXPos = a.value().find ("x");
				if (iXPos)	{
					iSizeWidth  = a.value().left(iXPos).toInt();
					iSizeHeight = a.value().right(a.value().length()-iXPos-1).toInt();
					if (iSizeWidth && iSizeHeight)
						p->sizeThumbnail = QSize (iSizeWidth, iSizeHeight);
				}

				a = childElem.attributeNode ( ATTRIB_DND_BUTTON );
				// The size is something like 50x50 so we ought to split
				iXPos = a.value().find ("x");
				if (iXPos)	{
					iSizeWidth  = a.value().left(iXPos).toInt();
					iSizeHeight = a.value().right(a.value().length()-iXPos-1).toInt();
					if (iSizeWidth && iSizeHeight)
						p->sizeButton = QSize (iSizeWidth, iSizeHeight);
				}
				a = childElem.attributeNode ( ATTRIB_DND_BUTTON_TYPE );
				p->bImageButton = a.value().toInt();

				a = childElem.attributeNode ( ATTRIB_DND_WITH_TEXT );
				p->bWithText = a.value().toInt();

				a = childElem.attributeNode ( ATTRIB_DND_TEXT_POS );
				p->iTextPosition = a.value().toInt();

				a = childElem.attributeNode ( ATTRIB_DND_START );
				if (!a.value().isNull())
					p->timeStartOffset = QTime::fromString(a.value());

				a = childElem.attributeNode ( ATTRIB_DND_DURATION );
				if (!a.value().isNull())
					p->timeDuration = QTime::fromString (a.value());

				a = childElem.attributeNode ( ATTRIB_DND_COLOR );
				p->colorForeground.setNamedColor(a.value());

				a = childElem.attributeNode ( ATTRIB_DND_START_WHEN );
				p->bStartAfterDropEvent = true;
				if (a.value() == QString ("false"))
					p->bStartAfterDropEvent = false;

				a = childElem.attributeNode ( ATTRIB_DND_NICE );
				p->iHowNiceShouldIBe = a.value().toInt();

				QDomElement fontElem = xmlExecuteInterfaceNode.toElement();
				if (fontElem.firstChild().toElement().tagName() == QString(TAG_FONT))
					p->font.fromString(fontElem.text());
			}
		}
		// Get the next "ExecuteInterface"
		xmlExecuteInterfaceNode = xmlExecuteInterfaceNode.nextSibling();
	}
	// Okay, next we sort out the colors ...
	for (t=0;t<m_arrayColors.count();t++)
		m_arrayColors[listOrder[t]]->setNamedColor(listColors[t]);

	// Next we take care that the tools we loaded are a tight fit for this program on this computer with the version of QDVDAuthor.
	mergeToolsPaths ( listToolsPaths );
	// Okay, we can get rid of all entries in the list we got form the ini - file ...
	for (t=0;t<listToolsPaths.count();t++)
		delete listToolsPaths[t];

	// Next in order to create the dvdauthor, and burn - block we need some data in the keywords to be used in resolvString()
	m_listKeywords.clear();
	m_listKeywords.append (m_qsDvdauthorFile);
	m_listKeywords.append (m_qsProjectPath);
	m_listKeywords.append (m_qsTempPath);

	// Prep the convertInterface for Movie / Audio conversion ...
	m_convertInterface.qsBlockName = resolvString(m_convertMenuBlock.qsBlockName);
	m_convertInterface.backgroundColor = m_convertMenuBlock.colorBackground;
	for (t=0;t<m_convertMenuBlock.blockOthers.listComments.count();t++)
		m_convertInterface.commentList.append(resolvString(m_convertMenuBlock.blockOthers.listComments[t]));
	for (t=0;t<m_convertMenuBlock.blockOthers.listCommands.count();t++)
		listCommands.append (resolvString(m_convertMenuBlock.blockOthers.listCommands[t]));
	m_convertInterface.listCommandList.append(listCommands);
	m_convertInterface.listDefaults.append(m_convertMenuBlock.blockOthers.iDefaultCommand);
	m_convertInterface.bShowBlock = !m_convertMenuBlock.bHidden;
	m_convertInterface.listBlockIDs.append (m_convertMenuBlock.blockEmptySound.iBlockID);
	
	listCommands.clear ();
	for (t=0;t<m_convertMenuBlock.blockEmptySound.listComments.count();t++)
		m_convertInterface.commentList.append(resolvString(m_convertMenuBlock.blockEmptySound.listComments[t]));
	for (t=0;t<m_convertMenuBlock.blockEmptySound.listCommands.count();t++)
		listCommands.append (resolvString(m_convertMenuBlock.blockEmptySound.listCommands[t]));
	m_convertInterface.listCommandList.append(listCommands);
	m_convertInterface.listDefaults.append(m_convertMenuBlock.blockEmptySound.iDefaultCommand);
	m_convertInterface.bShowBlock = !m_convertMenuBlock.bHidden;
	m_convertInterface.listBlockIDs.append (m_convertMenuBlock.blockEmptySound.iBlockID);

	listCommands.clear ();
	// And finally we get the information for the dvdauthor block, and the burning block ...
	m_dvdauthorInterface.qsBlockName = resolvString(m_dvdauthorMenuBlock.qsBlockName);
	m_dvdauthorInterface.backgroundColor = m_dvdauthorMenuBlock.colorBackground;
	for (t=0;t<m_dvdauthorMenuBlock.blockOthers.listComments.count();t++)
		m_dvdauthorInterface.commentList.append(resolvString(m_dvdauthorMenuBlock.blockOthers.listComments[t]));
	for (t=0;t<m_dvdauthorMenuBlock.blockOthers.listCommands.count();t++)
		listCommands.append (resolvString(m_dvdauthorMenuBlock.blockOthers.listCommands[t]));
	m_dvdauthorInterface.listCommandList.append(listCommands);
	m_dvdauthorInterface.listDefaults.append(m_dvdauthorMenuBlock.blockOthers.iDefaultCommand);
	m_dvdauthorInterface.bShowBlock = !m_dvdauthorMenuBlock.bHidden;
	m_dvdauthorInterface.listBlockIDs.append (m_dvdauthorMenuBlock.blockOthers.iBlockID);

	listCommands.clear ();
	m_burnInterface.qsBlockName = resolvString(m_burningMenuBlock.qsBlockName);
	m_burnInterface.backgroundColor = m_burningMenuBlock.colorBackground;
	for (uint i=0;i<m_burningMenuBlock.blockOthers.listSubBlocks.count();i++)	{
		for (t=0;t<m_burningMenuBlock.blockOthers.listSubBlocks[i]->listComments.count();t++)
			m_burnInterface.commentList.append(resolvString(m_burningMenuBlock.blockOthers.listSubBlocks[i]->listComments[t]));
		for (t=0;t<m_burningMenuBlock.blockOthers.listSubBlocks[i]->listCommands.count();t++)
			listCommands.append (resolvString(m_burningMenuBlock.blockOthers.listSubBlocks[i]->listCommands[t]));
	}
	m_burnInterface.listCommandList.append(listCommands);
	m_burnInterface.listDefaults.append(m_burningMenuBlock.blockOthers.iDefaultCommand);
	m_burnInterface.bShowBlock = !m_burningMenuBlock.bHidden;
	m_burnInterface.listBlockIDs.append (m_burningMenuBlock.blockOthers.iBlockID);

	return true;
}

bool QDVDAuthorInit::saveIniFile ()
{
	uint t;
	// This function writes in the xml - init - file for 'Q' DVD-Author.
	// the input for the file is taken from the structure stored in this class
	// and alsoo from the uses input.
	
	// Assign the file
	QString qsInitFile = QDir::homeDirPath();
	qsInitFile.append(INIT_DIR_NAME);
	qsInitFile.append(INIT_FILE_NAME);

	QFile initFile(qsInitFile);
	if (!initFile.open(IO_WriteOnly))
		return false;

	QDomDocument xmlDoc( INIT_DOCTYPE );
	// Okay the document is set, now let us write into the document ...
	QDomElement rootInit = xmlDoc.createElement( TAG_INIT );	// <QDVDAuthorInitFile>
	rootInit.setAttribute( ATTRIB_VERSION, QString ("%1").arg((float)INIT_VERSION));
	QString qsVisibleRegion ("false");
	if (m_bVisibleRegion)
		qsVisibleRegion = QString("true");
	rootInit.setAttribute( ATTRIB_VISIBLE_REGION, qsVisibleRegion);
	rootInit.setAttribute( ATTRIB_MAX_HISTORY, QString ("%1").arg(m_iMaxHistoryFiles));
	rootInit.setAttribute( ATTRIB_AUTOSAVE, QString ("%1").arg(m_iAutosave));
	xmlDoc.appendChild( rootInit );
	// First things first ... the color table ...
	QDomElement preferredElement = xmlDoc.createElement( TAG_PREFERRED_ENGINE );	// <PreferredEngine>
	QDomText domText = xmlDoc.createTextNode (MediaCreator::preferredEngine());
	preferredElement.appendChild( domText );
	rootInit.appendChild ( preferredElement );
	
	for (t=0;t<m_arrayColors.count();t++)	{
		QDomElement colorElement = xmlDoc.createElement( TAG_COLOR );	// <Color>
		domText = xmlDoc.createTextNode (m_arrayColors[t]->name());
		colorElement.setAttribute( ATTRIB_NUMBER, QString ("%1").arg(t));
		colorElement.appendChild( domText );
		rootInit.appendChild ( colorElement );
	}
	for (t=0;t<m_listToolsPaths.count();t++)	{
		QDomElement pathElement = xmlDoc.createElement( TAG_PATH );	// <Path>
		domText = xmlDoc.createTextNode (m_listToolsPaths[t]->qsFullPath);
		pathElement.setAttribute( ATTRIB_NAME, m_listToolsPaths[t]->qsExecutableName);
		pathElement.appendChild( domText );
		rootInit.appendChild ( pathElement );
	}
	for (t=0;t<m_listHistory.count();t++)	{
		QString qsFileName = QDir::homeDirPath() + QString (INIT_DIR_NAME) + QString ("history%1.png").arg (t);
		m_listHistoryPix[t]->save(qsFileName, "PNG", 100);
		QDomElement historyElement = xmlDoc.createElement( TAG_HISTORY );	// <History>
		domText = xmlDoc.createTextNode (qsFileName);
		historyElement.setAttribute( ATTRIB_NAME, m_listHistory[t]);
		historyElement.appendChild( domText );
		rootInit.appendChild ( historyElement );
/* The original code when we had the images stored in the ini file itself.
		Utils theUtil;
		QImage theImage = m_listHistoryPix[t]->convertToImage();
		QDomElement historyElement = xmlDoc.createElement( TAG_HISTORY );	// <History>
		domText = xmlDoc.createTextNode (theUtil.convertImageToString(theImage));
		historyElement.setAttribute( ATTRIB_NAME, m_listHistory[t]);
		historyElement.appendChild( domText );
		rootInit.appendChild ( historyElement );
*/
	}
	// Here we add the DragNDrop - stuff to the init - file ...
	DragNDropContainer *p = m_pDragNDropContainer;
	QDomElement dndElement  = xmlDoc.createElement( TAG_DRAGNDROP );  // <DragNDrop>
	QDomElement fontElement = xmlDoc.createElement( TAG_FONT );       // <Font>
	domText = xmlDoc.createTextNode (p->font.toString());
	dndElement.setAttribute( ATTRIB_DND_THUMBNAIL  , QString ("%1x%2").arg(p->sizeThumbnail.width()).arg(p->sizeThumbnail.height()));
	dndElement.setAttribute( ATTRIB_DND_BUTTON     , QString ("%1x%2").arg(p->sizeButton.width()).arg(p->sizeButton.height()));
	dndElement.setAttribute( ATTRIB_DND_BUTTON_TYPE, QString ("%1").arg(p->bImageButton));
	dndElement.setAttribute( ATTRIB_DND_WITH_TEXT  , QString ("%1").arg(p->bWithText));
	dndElement.setAttribute( ATTRIB_DND_TEXT_POS   , QString ("%1").arg(p->iTextPosition));
	dndElement.setAttribute( ATTRIB_DND_START      , QString ("%1").arg(p->timeStartOffset.toString(Qt::TextDate)));
	dndElement.setAttribute( ATTRIB_DND_DURATION   , QString ("%1").arg(p->timeDuration.toString(Qt::TextDate)));
	dndElement.setAttribute( ATTRIB_DND_COLOR      , QString ("%1").arg(p->colorForeground.name()));
	dndElement.setAttribute( ATTRIB_DND_START_WHEN , QString ("%1").arg(p->bStartAfterDropEvent ? "true" : "false"));
	dndElement.setAttribute( ATTRIB_DND_NICE       , QString ("%1").arg(p->iHowNiceShouldIBe));
	
	fontElement.appendChild( domText );
	dndElement.appendChild ( fontElement );
	rootInit.appendChild   ( dndElement );

	// Next are the ExecuteInterface's The menus go first bien sur ...
	QDomElement convertElement = xmlDoc.createElement( TAG_EXECUTE );	// <ExecuteInterface>
	saveMenuBlock (&m_convertMenuBlock,  &convertElement);
	rootInit.appendChild ( convertElement );

	// Next are the ExecuteInterface's The menus go first bien sur ...
	QDomElement executeElement = xmlDoc.createElement( TAG_EXECUTE );	// <ExecuteInterface>
	saveMenuBlock (&m_palMenuBlock,  &executeElement);
	saveMenuBlock (&m_ntscMenuBlock, &executeElement);
	rootInit.appendChild ( executeElement );
	
	QDomElement dvdauthorElement = xmlDoc.createElement( TAG_EXECUTE );	// <ExecuteInterface>
	saveMenuBlock (&m_dvdauthorMenuBlock, &dvdauthorElement);
	rootInit.appendChild ( dvdauthorElement );

	QDomElement burnElement = xmlDoc.createElement( TAG_EXECUTE );	// <ExecuteInterface>
	saveMenuBlock (&m_burningMenuBlock, &burnElement);
	rootInit.appendChild ( burnElement );

	// And finally we will store the generated string into the init file ...
	QString xml = xmlDoc.toString ();
//printf ("%s", (const char *)xml);
	initFile.writeBlock(xml, qstrlen (xml));

	initFile.close();
	return true;
}

bool QDVDAuthorInit::readMenuBlock (MenuBlock *pMenuBlock, QDomElement *pElem)
{
	QString qsBlockName;
	MenuBlock::Block *pBlock;
	// This function reads in a MenuBlock (with multiple sub-Block)
	// First we get the attributes ...
	QDomAttr a = pElem->attributeNode ( ATTRIB_BLOCKNAME );
	pMenuBlock->qsBlockName = a.value();
	a = pElem->attributeNode ( ATTRIB_COLOR );
	pMenuBlock->colorBackground.setNamedColor(a.value());
	// Next we parse through the associated Blocks.
	QDomNode xmlNode = pElem->firstChild ();
	while (!xmlNode.isNull())	{
		QDomElement searchTree = xmlNode.toElement();
		if (searchTree.tagName() == QString (TAG_BLOCK) )	{
			pBlock = &pMenuBlock->blockOthers;
			// get the name of this block ...
			qsBlockName = searchTree.attributeNode ( ATTRIB_NAME ).value();
			if (qsBlockName == QString (BLOCK_MOVIE_BACKGROUND))
				pBlock = &pMenuBlock->blockMovieBackground;
			else if  (qsBlockName == QString (BLOCK_IMAGE_BACKGROUND))
				pBlock = &pMenuBlock->blockImageBackground;
			else if  (qsBlockName == QString (BLOCK_EMPTY_SOUND))
				pBlock = &pMenuBlock->blockEmptySound;
			else if  (qsBlockName == QString (BLOCK_MPLEX))
				pBlock = &pMenuBlock->blockMplex;
			else if  (qsBlockName == QString (BLOCK_SPUMUX))
				pBlock = &pMenuBlock->blockSpumux;
			else if  (qsBlockName == QString (BLOCK_MOVIE_CONVERT))
				pBlock = &pMenuBlock->blockOthers;
			else if  (qsBlockName == QString (BLOCK_AUDIO_CONVERT))
				pBlock = &pMenuBlock->blockEmptySound;
			// and now we dwelve into reading the rest of the fest ...
			if (!readBlock (pBlock, &searchTree))	{
				return false;
			}
		}
		xmlNode = xmlNode.nextSibling();
	}
	return true;
}

bool QDVDAuthorInit::readBlock (MenuBlock::Block *pBlock, QDomElement *pElem)
{
	int iCommandNumber = 0;
	MenuBlock::Block *pSubBlock;
	// This function gets the information from Command / Comment - tags
	QDomNode xmlNode = pElem->firstChild();
	while (!xmlNode.isNull())	{
		QDomElement searchTree = xmlNode.toElement();
		if (searchTree.tagName() == QString (TAG_COMMAND))	{
			QDomAttr a = searchTree.attributeNode(ATTRIB_DEFAULT);
			if (a.value() == QString ("true"))
				pBlock->iDefaultCommand = iCommandNumber;
			pBlock->listCommands.append (searchTree.text());
			iCommandNumber ++;
			pBlock->iBlockID = m_iCurrentBlockID ++;
		}
		else if (searchTree.tagName() == QString (TAG_COMMENT)) 	{
			pBlock->listComments.append(searchTree.text());
			pBlock->iBlockID = m_iCurrentBlockID ++;
		}
		else if (searchTree.tagName() == QString (TAG_SUBBLOCK)) 	{
			pBlock->iBlockID = m_iCurrentBlockID ++;

			QDomNode subNode = searchTree.firstChild();
			pSubBlock = new MenuBlock::Block;
			// and now we dwelve into reading the rest of the fest ...
			QDomElement subTree = subNode.toElement();
			QDomElement subSubTree = subTree.firstChild().toElement();
			QDomElement *pReadElement = &searchTree;
			// Here we check if the SubBlock has another BLOCK inside.
			// This is due to the recursive saveBlock function call.
			if (subTree.tagName () == QString (TAG_BLOCK))
				pReadElement = &subTree;
			if (!readBlock (pSubBlock, pReadElement))
				return false;
			// And after I read in the SubBlock, I can set the attributes.
			QDomAttr a = searchTree.attributeNode(ATTRIB_NUMBER);
//			pSubBlock->iBlockID = a.value().toInt();
			a = searchTree.attributeNode(ATTRIB_ACTIVE);
			if (a.value() == QString ("true"))
				pSubBlock->iActive = 1;
			else if (a.value() == QString ("false"))
				pSubBlock->iActive = 0;
			pBlock->listSubBlocks.append (pSubBlock);
			subNode = subNode.nextSibling();
		}
		else
			return false;
		xmlNode = xmlNode.nextSibling();
	}
 	return true;
}

bool QDVDAuthorInit::saveMenuBlock (MenuBlock *pMenuBlock, QDomElement *pElement)
{
	if (pMenuBlock->bHidden)
		pElement->setAttribute( ATTRIB_HIDDEN, QString ("true") );
	QDomDocument xmlDoc = pElement->ownerDocument();
	QDomElement systemElement = xmlDoc.createElement( pMenuBlock->qsMenuTag );
	systemElement.setAttribute( ATTRIB_BLOCKNAME, pMenuBlock->qsBlockName );
	if (!pMenuBlock->colorBackground.name().isEmpty())
		systemElement.setAttribute( ATTRIB_COLOR, pMenuBlock->colorBackground.name() );
	
	if (pMenuBlock->qsMenuTag == QString (TAG_CONVERT))	{
		saveBlock (&pMenuBlock->blockEmptySound, &systemElement);   // Handling the sound conversion
		saveBlock (&pMenuBlock->blockOthers, &systemElement);       // Handling the movie conversion 
	}
	else if ( (pMenuBlock->qsMenuTag == QString (TAG_PAL)) || (pMenuBlock->qsMenuTag == QString (TAG_NTSC)) )	{
		saveBlock (&pMenuBlock->blockMovieBackground, &systemElement);
		saveBlock (&pMenuBlock->blockImageBackground, &systemElement);
		saveBlock (&pMenuBlock->blockEmptySound, &systemElement);
		saveBlock (&pMenuBlock->blockMplex, &systemElement);
		saveBlock (&pMenuBlock->blockSpumux, &systemElement);
	}
	else
		saveBlock (&pMenuBlock->blockOthers, &systemElement);
		
	pElement->appendChild (systemElement);
	
	return true;
}

bool QDVDAuthorInit::saveBlock (MenuBlock::Block *pBlock, QDomElement *pElement)
{
	uint t;
	QDomDocument xmlDoc = pElement->ownerDocument();
	QDomElement blockElement = xmlDoc.createElement( TAG_BLOCK );
	blockElement.setAttribute ( ATTRIB_NAME, pBlock->qsBlockTag );
	for (t=0;t<pBlock->listComments.count();t++)	{
		QDomElement commentElement = xmlDoc.createElement( TAG_COMMENT );
		QDomText commentText = xmlDoc.createTextNode (pBlock->listComments[t]);
		commentElement.appendChild ( commentText );
		blockElement.appendChild ( commentElement );
	}
	for (t=0;t<pBlock->listCommands.count();t++)	{
		QDomElement commandElement = xmlDoc.createElement( TAG_COMMAND );
		if (pBlock->iDefaultCommand == (int)t)
			commandElement.setAttribute( ATTRIB_DEFAULT, QString ("true"));
		QDomText commandText = xmlDoc.createTextNode (pBlock->listCommands[t]);
		commandElement.appendChild ( commandText );
		blockElement.appendChild ( commandElement );
	}
	for (t=0;t<pBlock->listSubBlocks.count();t++)	{
		QDomElement subblockElement = xmlDoc.createElement( TAG_SUBBLOCK );
		subblockElement.setAttribute( ATTRIB_NUMBER, QString ("%1").arg(t));
		if      (pBlock->listSubBlocks[t]->iActive ==  0)
			subblockElement.setAttribute( ATTRIB_ACTIVE, QString ("false"));
		else if (pBlock->listSubBlocks[t]->iActive ==  1)
			subblockElement.setAttribute( ATTRIB_ACTIVE, QString ("true"));
		else if (pBlock->listSubBlocks[t]->iActive == -1) ; // Do nothing in this case
		// hey time to get recursive
		if (!saveBlock(pBlock->listSubBlocks[t] , &subblockElement) )
			return false;
		blockElement.appendChild ( subblockElement );
	}
	pElement->appendChild ( blockElement );

	return true;
}

QValueList <ExecuteInterface *>&QDVDAuthorInit::getList()
{
	return m_listExecute;
}

void QDVDAuthorInit::setKeywords (QValueList<QString> listKeywords)
{
	m_listKeywords.clear();
	m_listKeywords = listKeywords;
}

void QDVDAuthorInit::setDefault(int iBlockID, uint iDefaultCommand)
{
	// This function will search through the MenuBlocks to find the appropriate comment 
	// in order to set the default command correctly.
	getBlock(iBlockID)->iDefaultCommand = iDefaultCommand;
}

QDVDAuthorInit::MenuBlock::Block *QDVDAuthorInit::getBlock (int iBlockID)
{
	uint t;
	MenuBlock *arrayMenuBlocks[4] = {&m_palMenuBlock, &m_ntscMenuBlock, &m_dvdauthorMenuBlock, &m_burningMenuBlock};
	MenuBlock *pMenuBlock;
	MenuBlock::Block *pReturn = NULL;
	for (t=0;t<4;t++)	{
		pMenuBlock = arrayMenuBlocks[t];
		if (pMenuBlock->blockMovieBackground.iBlockID == iBlockID)
			 pReturn = &pMenuBlock->blockMovieBackground;
		else if (pMenuBlock->blockImageBackground.iBlockID == iBlockID)
			 pReturn = &pMenuBlock->blockImageBackground;
		else if (pMenuBlock->blockEmptySound.iBlockID == iBlockID)
			 pReturn = &pMenuBlock->blockEmptySound;
		else if (pMenuBlock->blockMplex.iBlockID == iBlockID)
			 pReturn = &pMenuBlock->blockMplex;
		else if (pMenuBlock->blockSpumux.iBlockID == iBlockID)
			 pReturn = &pMenuBlock->blockSpumux;
		else if (pMenuBlock->blockOthers.iBlockID == iBlockID)
			 pReturn = &pMenuBlock->blockOthers;
		else 	{	// No luck finding the block, let us check in the subBlocks of the MovieBackground - block 
			for (uint i=0;i<pMenuBlock->blockMovieBackground.listSubBlocks.count();i++)	{
				if (pMenuBlock->blockMovieBackground.listSubBlocks[i]->iBlockID == iBlockID)
					pReturn = pMenuBlock->blockMovieBackground.listSubBlocks[i];
			}
		}
		if (pReturn)
			return pReturn;
	}
	return NULL;
}

void QDVDAuthorInit::appendPreProcessing ()
{
	int t;
	// First we clear the 'old' list ...
	if (m_listExecute.count() > 2)	{
		for (t=0;t<(int)m_listExecute.count();t++)
			delete m_listExecute[t];
	}
	m_listExecute.clear();

	// TODO: 
	// Then we append pre processing (movie conversion if neccesary).
//	m_listExecute.append(&m_convertInterface);
}

void QDVDAuthorInit::appendPostProcessing ()
{
	m_listExecute.append(&m_dvdauthorInterface);
	m_listExecute.append(&m_burnInterface);
}

void QDVDAuthorInit::appendMenu (DVDMenu *pMenu, QString qsMenuName, QString qsBackgroundFileName)
{
	// This function will generate the Commands needed per Menu.
	// The information comes from the menu on the one hand and on the other 
	// it comes from the init - file. This is the mating ground ...
	uint t, i;
	MenuBlock *pMenuBlock;
	ExecuteInterface *pInterface = new ExecuteInterface;
	
	// First things first, here we determine the Format ...
	// iFormat > 4 equals PAL, otherwise NTSC
	int iFormat = pMenu->getFormat (pMenu->getInterface()->iWidth, pMenu->getInterface()->iHeight);

	// Next is to create the keyword list ... needed for resolvString() ...
	m_listKeywords.clear();
	m_listKeywords.append(m_qsDvdauthorFile);
	m_listKeywords.append(m_qsProjectPath);
	m_listKeywords.append(m_qsTempPath);
	m_listKeywords.append (QString ("%1").arg(pMenu->getInterface()->iWidth));
	m_listKeywords.append (QString ("%1").arg(pMenu->getInterface()->iHeight));

	if (iFormat > 4)	{	// PAL
		pMenuBlock = &m_palMenuBlock;
		m_listKeywords.append ("PAL");
	}
	else	{
		pMenuBlock = &m_ntscMenuBlock;
		m_listKeywords.append ("NTSC");
	}
	QFileInfo fileInfo (qsBackgroundFileName);
	m_listKeywords.append(pMenu->name());        // +MENU_NAME+
	m_listKeywords.append(pMenu->getInterface()->timeOffset.toString  ()); // +START_POS+
	m_listKeywords.append(pMenu->getInterface()->timeDuration.toString()); // +DURATION+
	m_listKeywords.append(fileInfo.fileName() ); // +BACKGROUND_FILE_NAME+
	m_listKeywords.append(qsBackgroundFileName); // +BACKGROUND_FULL_NAME+
	m_listKeywords.append(m_qsTempPath);         // +SOUND_LIST+

	// Sanity check ...
//	if ( (!pMenuBlock) || (!pMenuBlock->bValid) )
	if (!pMenuBlock)
		return;

	pInterface->qsBlockName = resolvString (pMenuBlock->qsBlockName);
//	pInterface->backgroundColor = pMenuBlock->colorBackground;
	if (m_arrayColors.count() > 0)
		pInterface->backgroundColor = *m_arrayColors[m_iColorCounter++];
	else
		pInterface->backgroundColor.setNamedColor("#9EEE81");

	if (m_iColorCounter > (int)m_arrayColors.count())
		m_iColorCounter = 0;

	// Here we check if the background is a movie or an image.
	QFileInfo backgroundInfo (qsBackgroundFileName);
	MenuBlock::Block *pBlock = &pMenuBlock->blockImageBackground;
	     
	MediaInfo *pMediaInfo = pMenu->getDVDAuthor()->getMediaInfo ();
	QString qsExtensions  = pMediaInfo->getExtensions (true);
	if (qsExtensions.find (QString (".")+backgroundInfo.extension()) > -1)	{
		// Okay we have a movieBackground. Now every frame of that background has to be rendered
		// so we need to run quite some commands in here ...
		// Fortunately all we need is stored in the blockMovieBackground structure.
		for (t=0;t<pMenuBlock->blockMovieBackground.listSubBlocks.count();t++)
			addBlock (pInterface, pMenuBlock->blockMovieBackground.listSubBlocks[t]);
		// After we have the movie extracted, we check if the user added sound files
		if ( (pMenu->getInterface()->bMovieBackgroundHasSound) && (pMenu->getInterface()->listSoundEntries.count() == 0) )	{
			QString qsBackgroundAudio = QString ("%1/%2/clip.wav").arg(qsMenuName).arg(backgroundInfo.fileName ());
			backgroundInfo.setFile(qsBackgroundAudio);
			pMenu->getInterface()->listSoundEntries.append (qsBackgroundAudio);
		}
	}
	else if (pMenu->isMovieMenu ())	{
		// The difference here is that the background is a image
		// So we need everything from the point of the rendering of the menu.
		bool bAddBlocks = false;
		for (t=0;t<pMenuBlock->blockMovieBackground.listSubBlocks.count();t++)	{
			MenuBlock::Block *pSubBlock = pMenuBlock->blockMovieBackground.listSubBlocks[t];
			if (!bAddBlocks) {  // Only need to check up till we find the render command
				for (i=0;i<pSubBlock->listCommands.count();i++)	{
					// Check every command for the RENDER_MENU - string
					if (pSubBlock->listCommands[i].find( QString(RENDER_MENU)) > -1)
						bAddBlocks = true;
				}
			}
			if (bAddBlocks)
				addBlock (pInterface, pMenuBlock->blockMovieBackground.listSubBlocks[t]);
		}
	}
	else 
		addBlock (pInterface, pBlock);

	// Note, spumux > button 'label' is ought to be 'name' now ??? (spumux from 19 Feb 2004)
	m_soundList.clear();
	if (pMenu->getInterface()->listSoundEntries.count() == 0)	{
		// This is the only instance where we need a hardcoded naming scheme ...
		m_soundList.append (QString ("\"%1/menu.mp2\"").arg(qsMenuName));
		m_listKeywords[MAX_KEYWORDS - 1] = m_soundList[0];
		addBlock(pInterface, &pMenuBlock->blockEmptySound);
	}
	else	{
		QString qsConvertedSound;
		for (t=0;t<pMenu->getInterface()->listSoundEntries.count();t++)	{
			QString qsEntryName = pMenu->getInterface()->listSoundEntries[t];
			SourceFileEntry *pEntry = pMenu->getDVDAuthor()->getSourceEntryByDisplayName(qsEntryName);
			if (!pEntry)
				continue;
			for (i=0;i<pEntry->listFileInfos.count();i++)	{
				qsConvertedSound = convertSoundFile (pEntry->listFileInfos[i]->qsFileName);
				m_soundList.append (QString ("\"%1\"").arg(qsConvertedSound));
			}
		}
		m_listKeywords[MAX_KEYWORDS - 1] = m_soundList.join (" ");
	}
	addBlock(pInterface, &pMenuBlock->blockMplex);
	addBlock(pInterface, &pMenuBlock->blockSpumux);

	pInterface->bShowBlock = !pMenuBlock->bHidden;
	pInterface->listKeywords = m_listKeywords;
	m_listExecute.append(pInterface);
}

void QDVDAuthorInit::addBlock(ExecuteInterface *pInterface, MenuBlock::Block *pBlock)
{
	uint t;
	QStringList stringList;
	// Currently not supported more then one comment per block ...
//	for (t=0;t<pBlock->listComments.count(); stringList.append(resolvString(pMenu, pBlock->listComments[t++])));
	pInterface->commentList.append(resolvString(pBlock->listComments[0])); //stringList);
//	stringList.clear();
	for (t=0;t<pBlock->listCommands.count(); stringList.append(resolvString(pBlock->listCommands[t++])));
	pInterface->listCommandList.append(stringList);
	pInterface->listDefaults.append (pBlock->iDefaultCommand);
	pInterface->listBlockIDs.append (pBlock->iBlockID);

	stringList.clear();
}

QString QDVDAuthorInit::resolvString (QString qsInput)
{
	// This function resolves the variables in the stored commands / commenst / names like
	// +MENU_NAME+, +WIDTH+, +BACKGROUND_FILE_NAME+ etc.
	QString qsOutput = qsInput;
	uint t;
	char cArrayKeyword[MAX_KEYWORDS][35] = {"+DVDAUTHOR_XML+", "+PROJECT_PATH+", "+TEMP_PATH+", "+WIDTH+", "+HEIGHT+",
		"+FORMAT+", "+MENU_NAME+", "+START_POS+", "+DURATION+", "+BACKGROUND_FILE_NAME+", "+BACKGROUND_FULL_NAME+", "+SOUND_LIST+"};
	for (t=0;t<m_listKeywords.count();t++)	{
		// Here we replace the actual keyword with the real string.
//		if (qsOutput.find (cArrayKeyword[t]) )
			qsOutput.replace(cArrayKeyword[t], m_listKeywords[t]);
	}
	return qsOutput;
}

QString QDVDAuthorInit::convertSoundFile (QString qsOriginalSoundFile)
{
	// TODO : 
	// This function is supposed to detect the sound preferences of the sound files
	// and convert them to the propper mp2, pcm, or ac3 layer format for usage with the DVD
	QString qsReturn;

	qsReturn = qsOriginalSoundFile;

	return qsReturn;
}

void QDVDAuthorInit::resetInitFile ()
{
	QString qsInitFile = QDir::homeDirPath();
	qsInitFile.append(INIT_DIR_NAME);
	qsInitFile.append(INIT_FILE_NAME);
	
	QFile initFile(qsInitFile);
	if ( initFile.open( IO_WriteOnly ) )	{
		QTextStream stream( &initFile );
//		QString qsResetInitFile = QString ("<QDVDAuthorInitFile version=\"%1\">\n").arg(INIT_VERSION);
		QString qsResetInitFile = QString ("<QDVDAuthorInitFile version=\"%1\">\n").arg((float)INIT_VERSION);
		qsResetInitFile += QString (
"  <Color number=\"0\">#9EEE81</Color>\n"
"  <Color number=\"1\">#8FEED3</Color>\n"
"  <Color number=\"2\">#E0EE8F</Color>\n"
"  <Color number=\"3\">#ED97EE</Color>\n"
"  <Color number=\"4\">#9EEE81</Color>\n"
"  <Color number=\"5\">#8FEED3</Color>\n"
"  <Color number=\"6\">#E0EE8F</Color>\n"
"  <Color number=\"7\">#ED97EE</Color>\n"
"  <Color number=\"8\">#9EEE81</Color>\n"
"  <Color number=\"9\">#8FEED3</Color>\n"
"  <Color number=\"10\">#E0EE8F</Color>\n"
"  <Color number=\"11\">#F92F5E</Color>\n"
"  <DragNDrop start_offset=\"00:00:00\" thumbnail_size=\"50x50\" button_type=\"1\" text_pos=\"1\" duration=\"00:00:00\" color=\"#ff0000\" with_text=\"1\" button_size=\"150x150\" >\n"
"    <Font>Sans Serif,24,-1,5,75,0,0,0,0,0</Font>\n"
"  </DragNDrop>\n"
"  <ExecuteInterface>\n"
"    <Convert blockname=\"Conversion Block\" color=\"#F92F5E\">\n"
"      <Block name=\"MOVIE_CONVERT\">\n"
"        <Comment><p align=\"center\">This will convert the input movie file from DV - format into DVD conform material.</p></Comment>\n"
"        <Command>transcode -i \"mindfields_low_osx.avi\" -x ffmpeg -V -w 8500 --encode_fields b --export_asr 2 -F 8,-c -q 6 -4 2 -2 1 -N 0.5 -E -10 -K tmpgenc -R 2 -y mpeg2enc,mp2enc -b 224 -m \"mindfields_out_dvd.mpg\" -o \"mindfields_out_dvd.mpg\" --print_status 10</Command>\n"
"        <Comment><p align=\"center\">This will convert the input movie file from mpeg1 - format into DVD conform material.</p></Comment>\n"
"        <Command>transcode -i \"mindfields_low_osx.avi\" -x ffmpeg -V -w 8500 --encode_fields b --export_asr 2 -F 8,-c -q 6 -4 2 -2 1 -N 0.5 -E -10 -K tmpgenc -R 2 -y mpeg2enc,mp2enc -b 224 -m \"mindfields_out_dvd.mpg\" -o \"mindfields_out_dvd.mpg\" --print_status 10</Command>\n"
"      </Block>\n"
"      <Block name=\"AUDIO_CONVERT\">\n"
"        <Comment><p align=\"center\">We need to convert from OGG-Vorbis format to mp2.</p></Comment>\n"
"        <Command>oggdec \"test.ogg\" -o - | mp2enc -o \"test2.mp2\"</Command>\n"
"        <Comment><p align=\"center\">We need to convert from mp3 format to mp2.</p></Comment>\n"
"        <Command>lame --decode \"blues.mp3\" - | mp2enc -o \"blues.mp2\"</Command>\n"
"        <Command>mplayer \"mymusicfile.mp3\" -vo null -ao pcm -aofile \"menu_audio.pcm\" ; mp2enc -r 48000 -o \"menu_audio.mpa\" &lt; \"menu_audio.pcm\"</Command>\n"
"      </Block>\n"
"    </Convert>\n"
"  </ExecuteInterface>\n"
"  <ExecuteInterface>\n"
"    <Pal blockname=\"+MENU_NAME+ (PAL +WIDTH+x+HEIGHT+)\">\n"
"      <Block name=\"MOVIE_BACKGROUND\">\n"
"        <SubBlock number=\"1\" active=\"false\">\n"
"          <Comment>&lt;p align=\"center\"&gt;This line of command will clean up the source movie&lt;BR&gt;&lt;U&gt;Note :&lt;/U&gt;Might not be needed.&lt;/p&gt;</Comment>\n"
"          <Command default=\"true\">mkdir -p \"+TEMP_PATH+/+MENU_NAME+/background\";echo \"Skipping cleaning of source file\"</Command>\n"
"          <Command>mkdir -p \"+TEMP_PATH+/+MENU_NAME+/background\";mencoder -oac pcm -ovc lavc -o \"+TEMP_PATH+/+MENU_NAME+/background/clean.avi\" \"+BACKGROUND_FULL_NAME+\"</Command>\n"
"        </SubBlock>\n"
"        <SubBlock number=\"2\">\n"
"          <Comment>&lt;p align=\"center\"&gt;This Command will extract the timeframe of interest.&lt;/p&gt;</Comment>\n"
"          <Command default=\"true\">mencoder -oac pcm -ovc lavc -ss +START_POS+ -endpos +DURATION+ -o \"+TEMP_PATH+/+MENU_NAME+/background/clip.avi\" \"+TEMP_PATH+/+MENU_NAME+/background/clean.avi\"</Command>\n"
"        </SubBlock>\n"
"        <SubBlock number=\"3\">\n"
"          <Comment>&lt;p align=\"center\"&gt;Here we extract the audio part of the source background&lt;/p&gt;</Comment>\n"
"          <Command default=\"true\">mplayer -ao pcm -vo null -vc dummy -aofile \"+TEMP_PATH+/+MENU_NAME+/background/clip.wav\" \"+TEMP_PATH+/+MENU_NAME+/background/clip.avi\"</Command>\n"
"        </SubBlock>\n"
"        <SubBlock number=\"4\">\n"
"          <Comment>&lt;p align=\"center\"&gt;And now we extract the image files of the source movie&lt;/p&gt;</Comment>\n"
"          <Command default=\"true\">mplayer -ao null -vo jpeg:outdir=\"+TEMP_PATH+/+MENU_NAME+/background\" \"+TEMP_PATH+/+MENU_NAME+/background/clip.avi\"</Command>\n"
"        </SubBlock>\n"
"        <SubBlock number=\"6\">\n"
"          <Comment>&lt;p align=\"center\"&gt;And this is the place where we internally render the images&lt;/p&gt;</Comment>\n"
"          <Command default=\"true\" canmodify=\"false\">"RENDER_MENU"+MENU_NAME+</Command>\n"
"        </SubBlock>\n"
"        <SubBlock number=\"5\">\n"
"          <Comment>&lt;p align=\"center\"&gt;We can remove the big temp files&lt;/p&gt;</Comment>\n"
"          <Command default=\"true\">cd \"+TEMP_PATH+/+MENU_NAME+/\"; find . -name clip.avi -exec rm {} \\;; find . -name clean.avi -exec rm {} \\;</Command>\n"
"        </SubBlock>\n"
"        <SubBlock number=\"6\">"
"          <Comment>&lt;p align=\"center\"&gt;This line of command will convert the extracted frames &lt;BR&gt;into a MPEG2 stream in PAL format +WIDTH+x+HEIGHT+&lt;/p&gt;</Comment>\n"
"          <Command>png2yuv -j \"+TEMP_PATH+/+MENU_NAME+/background/rendered_%08d.png\" -I p -f 25 -b 1 | mpeg2enc -f 8 -n p -F 3 -M 1 -o \"+TEMP_PATH+/+MENU_NAME+/menu.m2v\"</Command>\n"
"        </SubBlock>"
"      </Block>\n"
"      <Block name=\"IMAGE_BACKGROUND\">\n"
"        <Comment>&lt;p align=\"center\"&gt;This line of command will convert the background image&lt;BR&gt;into a MPEG2 stream in PAL format +WIDTH+x+HEIGHT+&lt;/p&gt;</Comment>\n"
"        <Command>jpegtopnm \"+BACKGROUND_FULL_NAME+\" | ppmtoy4m -n 1 -F25:1 -A59:54 -I t -L | mpeg2enc -f 8 -n p -o \"+TEMP_PATH+/+MENU_NAME+/menu.m2v\"</Command>\n"
"        <Command default=\"true\">jpeg2yuv -n 50 -I p -f 25 -j \"+BACKGROUND_FULL_NAME+\" | mpeg2enc -n p -f 8 -o \"+TEMP_PATH+/+MENU_NAME+/menu.m2v\"</Command>\n"
"      </Block>\n"
"      <Block name=\"EMPTY_SOUND\">\n"
"        <Comment>&lt;p align=\"center\"&gt;This line creates an empty sound file to mix into the menu-movie.&lt;/p&gt;</Comment>\n"
"        <Command>arecord -f dat -twav -d 1 | mp2enc -r 48000 -o \"+TEMP_PATH+/+MENU_NAME+/menu.mp2\"</Command>\n"
"        <Command>arecord -S -M -t 1 -w /dev/stdout | mp2enc -r 48000 -o \"+TEMP_PATH+/+MENU_NAME+/menu.mp2\"</Command>\n"
"        <Command>sox -t raw -s -w -c 2 -r 48000 /dev/zero -t wav -c 2 -r 48000 /dev/stdout trim 0 1 | mp2enc -r 48000 -o \"+TEMP_PATH+/+MENU_NAME+/menu.mp2\"</Command>\n"
"        <Command default=\"true\">dd if=/dev/zero bs=4 count=1920 | toolame -b128 -s 48 /dev/stdin \"+TEMP_PATH+/+MENU_NAME+/menu.mp2\"</Command>\n"
"      </Block>\n"
"      <Block name=\"MPLEX\">\n"
"        <Comment>&lt;p align=\"center\"&gt;The following command will multiplex the sound file into the menu-movie.&lt;/p&gt;</Comment>\n"
"        <Command default=\"true\">mplex -f 8 -o \"+TEMP_PATH+/+MENU_NAME+/menu.mpg\" \"+TEMP_PATH+/+MENU_NAME+/menu.m2v\" +SOUND_LIST+</Command>\n"
"      </Block>\n"
"      <Block name=\"SPUMUX\">\n"
"        <Comment>&lt;p align=\"center\"&gt;Here we use spumux to add the subpicture (buttons) the the mpeg2 video.&lt;/p&gt;</Comment>\n"
"        <Command default=\"true\">spumux \"+TEMP_PATH+/+MENU_NAME+/menu.xml\" &lt; \"+TEMP_PATH+/+MENU_NAME+/menu.mpg\" &gt; \"+TEMP_PATH+/+MENU_NAME+_menu.mpg\"</Command>\n"
"      </Block>\n"
"    </Pal>\n"
"    <Ntsc blockname=\"+MENU_NAME+ (NTSC +WIDTH+x+HEIGHT+)\">\n"
"      <Block name=\"MOVIE_BACKGROUND\">\n"
"        <SubBlock number=\"1\" active=\"false\">\n"
"          <Comment>&lt;p align=\"center\"&gt;This line of command will clean up the source movie&lt;BR&gt;Might not be needed.&lt;/p&gt;</Comment>\n"
"          <Command default=\"true\">mkdir -p \"+TEMP_PATH+/+MENU_NAME+/background\";echo \"Skipping cleaning of source file\"</Command>\n"
"          <Command>mkdir -p \"+TEMP_PATH+/+MENU_NAME+/background\";mencoder -oac pcm -ovc lavc -o \"+TEMP_PATH+/+MENU_NAME+/background/clean.avi\" \"+BACKGROUND_FULL_NAME+\"</Command>\n"
"        </SubBlock>\n"
"        <SubBlock number=\"2\">\n"
"          <Comment>&lt;p align=\"center\"&gt;This Command will extract the requested timeframe.&lt;/p&gt;</Comment>\n"
"          <Command default=\"true\">mencoder -oac pcm -ovc lavc -ss +START_POS+ -endpos +DURATION+ -o \"+TEMP_PATH+/+MENU_NAME+/background/clip.avi\" \"+TEMP_PATH+/+MENU_NAME+/background/clean.avi\"</Command>\n"
"        </SubBlock>\n"
"        <SubBlock number=\"3\">\n"
"          <Comment>&lt;p align=\"center\"&gt;Here we extract the audio part of the source background&lt;/p&gt;</Comment>\n"
"          <Command default=\"true\">mplayer -ao pcm -vo null -vc dummy -aofile \"+TEMP_PATH+/+MENU_NAME+/background/clip.wav\" \"+TEMP_PATH+/+MENU_NAME+/background/clip.avi\"</Command>\n"
"        </SubBlock>\n"
"        <SubBlock number=\"4\">\n"
"          <Comment>&lt;p align=\"center\"&gt;And now we extract the image files of the source movie&lt;/p&gt;</Comment>\n"
"          <Command default=\"true\">mplayer -ao null -vo jpeg:outdir=\"+TEMP_PATH+/+MENU_NAME+/background\" \"+TEMP_PATH+/+MENU_NAME+/background/clip.avi\"</Command>\n"
"        </SubBlock>\n"
"        <SubBlock number=\"6\">\n"
"          <Comment>&lt;p align=\"center\"&gt;And this is the place where we internally render the images&lt;/p&gt;</Comment>\n"
"          <Command default=\"true\" canmodify=\"false\">"RENDER_MENU"+MENU_NAME+</Command>\n"
"        </SubBlock>\n"
"        <SubBlock number=\"5\">\n"
"          <Comment>&lt;p align=\"center\"&gt;We can remove the big temp files&lt;/p&gt;</Comment>\n"
"          <Command default=\"true\">cd \"+TEMP_PATH+/+MENU_NAME+/\"; find . -name clip.avi -exec rm {} \\;; find . -name clean.avi -exec rm {} \\;</Command>\n"
"        </SubBlock>\n"
"        <SubBlock number=\"6\">"
"          <Comment>&lt;p align=\"center\"&gt;This line of command will convert the extracted frames &lt;BR&gt;into a MPEG2 stream in NTSC format +WIDTH+x+HEIGHT+&lt;/p&gt;</Comment>\n"
"          <Command>png2yuv -j \"+TEMP_PATH+/+MENU_NAME+/background/rendered_%08d.png\" -I p -f 29.97 -b 1 | mpeg2enc -f 8 -n n -F 4 -M 1 -o \"+TEMP_PATH+/+MENU_NAME+/menu.m2v\"</Command>\n"
"        </SubBlock>"
"      </Block>\n"
"      <Block name=\"IMAGE_BACKGROUND\">\n"
"        <Comment>&lt;p align=\"center\"&gt;This line of command will convert the background image&lt;BR&gt;into a MPEG2 stream in NTSC format +WIDTH+x+HEIGHT+&lt;/p&gt;</Comment>\n"
"        <Command>jpegtopnm \"+BACKGROUND_FULL_NAME+\" | ppmtoy4m -n 1 -F30000:1001 -A10:11 -I t -L | mpeg2enc -f 8 -n n -o \"+TEMP_PATH+/+MENU_NAME+/menu.m2v\"</Command>\n"
"        <Command default=\"true\">jpeg2yuv -n 50 -I p -f 29.97 -j \"+BACKGROUND_FULL_NAME+\" | mpeg2enc -n n -f 8 -o \"+TEMP_PATH+/+MENU_NAME+/menu.m2v\"</Command>\n"
"      </Block>\n"
"      <Block name=\"EMPTY_SOUND\">\n"
"        <Comment>&lt;p align=\"center\"&gt;This line creates an empty sound file to mix into the menu-movie.&lt;/p&gt;</Comment>\n"
"        <Command default=\"true\">arecord -f dat -twav -d 1 /dev/stdout | mp2enc -r 48000 -o \"+TEMP_PATH+/+MENU_NAME+/menu.mp2\"</Command>\n"
"        <Command>arecord -S -M -t 1 -w /dev/stdout | mp2enc -r 48000 -o \"+TEMP_PATH+/+MENU_NAME+/menu.mp2\"</Command>\n"
"        <Command>sox -t raw -s -w -c 2 -r 48000 /dev/zero -t wav -c 2 -r 48000 /dev/stdout trim 0 1 | mp2enc -r 48000 -o \"+TEMP_PATH+/+MENU_NAME+/menu.mp2\"</Command>\n"
"        <Command>dd if=/dev/zero bs=4 count=1601.6 | toolame -b128 -s 48 /dev/stdin \"+TEMP_PATH+/+MENU_NAME+/menu.mp2\"</Command>\n"
"      </Block>\n"
"      <Block name=\"MPLEX\">\n"
"        <Comment>&lt;p align=\"center\"&gt;The following command will multiplex the sound file into the menu-movie.&lt;/p&gt;</Comment>\n"
"        <Command default=\"true\">mplex -f 8 -o \"+TEMP_PATH+/+MENU_NAME+/menu.mpg\" \"+TEMP_PATH+/+MENU_NAME+/menu.m2v\" +SOUND_LIST+</Command>\n"
"      </Block>\n"
"      <Block name=\"SPUMUX\">\n"
"        <Comment>&lt;p align=\"center\"&gt;Here we use spumux to add the subpicture (buttons) the the mpeg2 video.&lt;/p&gt;</Comment>\n"
"        <Command default=\"true\">spumux \"+TEMP_PATH+/+MENU_NAME+/menu.xml\" &lt; \"+TEMP_PATH+/+MENU_NAME+/menu.mpg\" &gt; \"+TEMP_PATH+/+MENU_NAME+_menu.mpg\"</Command>\n"
"      </Block>\n"
"    </Ntsc>\n"
"  </ExecuteInterface>\n"
"  <ExecuteInterface>\n"
"    <Dvdauthor blockname=\"dvdauthor\" color=\"#F92F5E\">\n"
"      <Block>\n"
"        <Comment>Here we start dvdauthor with the generated xml file.</Comment>\n"
"        <Command>dvdauthor -x \"+DVDAUTHOR_XML+\"</Command>\n"
"      </Block>\n"
"    </Dvdauthor>\n"
"  </ExecuteInterface>\n"
"  <ExecuteInterface hidden=\"true\">\n"
"    <Burning blockname=\"Burn DVD\" color=\"#E0EE8F\">\n"
"      <Block>\n"
"        <SubBlock number=\"1\" active=\"true\">\n"
"          <Comment>Last we burn the DVD.</Comment>\n"
"          <Command>growisofs -Z /dev/dvd -dvd-video +PROJECT_PATH+/</Command>\n"
"        </SubBlock>\n"
"        <SubBlock number=\"2\" active=\"false\">\n"
"          <Comment>This command will format a empty -RW DVD (only needed for older drives).</Comment>\n"
"          <Command>dvd+rw-format -f /dev/srcd0</Command>\n"
"        </SubBlock>\n"
"        <SubBlock number=\"3\" active=\"false\">\n"
"          <Comment>This command generates an iso image of the DVD.</Comment>\n"
"          <Command>mkisofs -dvd-video -udf -o +TEMP_PATH+/dvd.iso +PROJECT_PATH+/</Command>\n"
"        </SubBlock>\n"
"        <SubBlock number=\"4\" active=\"false\">\n"
"          <Comment>This command burns the iso DVD image to DVD</Comment>\n"
"          <Command>dvdrecord -dao speed=2 dev=0,0,0 +TEMP_PATH+/dvd.iso</Command>\n"
"        </SubBlock>\n"
"      </Block>\n"
"    </Burning>\n"
"  </ExecuteInterface>\n"
"</QDVDAuthorInitFile>\n");
  		stream << (const char *)qsResetInitFile << "\n";
		initFile.close();
	}
}

/*
void QDVDAuthorInit::resetInitFile ()
{
	QString qsInitFile = QDir::homeDirPath();
	qsInitFile.append(INIT_FILE_NAME);
	
	QFile initFile(qsInitFile);
	if ( initFile.open( IO_WriteOnly ) )	{
		QTextStream stream( &initFile );
//		QString qsResetInitFile = QString ("<QDVDAuthorInitFile version=\"%1\">\n").arg(INIT_VERSION);
		QString qsResetInitFile = QString ("\
<QDVDAuthorInitFile version=\"%1\" >\n\
  <Color number=\"0\">#9EEE81</Color>\n\
  <Color number=\"1\">#8FEED3</Color>\n\
  <Color number=\"2\">#E0EE8F</Color>\n\
  <Color number=\"3\">#ED97EE</Color>\n\
  <Color number=\"4\">#9EEE81</Color>\n\
  <Color number=\"5\">#8FEED3</Color>\n\
  <Color number=\"6\">#E0EE8F</Color>\n\
  <Color number=\"7\">#ED97EE</Color>\n\
  <Color number=\"8\">#9EEE81</Color>\n\
  <Color number=\"9\">#8FEED3</Color>\n\
  <Color number=\"10\">#E0EE8F</Color>\n\
  <Color number=\"11\">#F92F5E</Color>\n\
  <DragNDrop start_offset=\"00:00:00\" thumbnail_size=\"50x50\" button_type=\"1\" text_pos=\"1\" duration=\"00:00:00\" color=\"#ff0000\" with_text=\"1\" button_size=\"150x150\" >\n\
    <Font>Sans Serif,24,-1,5,75,0,0,0,0,0</Font>\n\
  </DragNDrop>\n\
  <ExecuteInterface>\n\
    <Convert blockname=\"Conversion Block\" color=\"#F92F5E\">\n\
      <Block name=\"MOVIE_CONVERT\">\n\
        <Comment><p align=\"center\">This will convert the input movie file from DV - format into DVD conform material.</p></Comment>\n\
        <Command>transcode -i \"mindfields_low_osx.avi\" -x ffmpeg -V -w 8500 --encode_fields b --export_asr 2 -F 8,-c -q 6 -4 2 -2 1 -N 0.5 -E -10 -K tmpgenc -R 2 -y mpeg2enc,mp2enc -b 224 -m \"mindfields_out_dvd.mpg\" -o \"mindfields_out_dvd.mpg\" --print_status 10</Command>\n\
        <Comment><p align=\"center\">This will convert the input movie file from mpeg1 - format into DVD conform material.</p></Comment>\n\
        <Command>transcode -i \"mindfields_low_osx.avi\" -x ffmpeg -V -w 8500 --encode_fields b --export_asr 2 -F 8,-c -q 6 -4 2 -2 1 -N 0.5 -E -10 -K tmpgenc -R 2 -y mpeg2enc,mp2enc -b 224 -m \"mindfields_out_dvd.mpg\" -o \"mindfields_out_dvd.mpg\" --print_status 10</Command>\n\
      </Block>\n\
      <Block name=\"AUDIO_CONVERT\">\n\
        <Comment><p align=\"center\">We need to convert from OGG-Vorbis format to mp2.</p></Comment>\n\
        <Command>oggdec \"test.ogg\" -o - | mp2enc -o \"test2.mp2\"</Command>\n\
        <Comment><p align=\"center\">We need to convert from mp3 format to mp2.</p></Comment>\n\
        <Command>lame --decode \"blues.mp3\" - | mp2enc -o \"blues.mp2\"</Command>\n\
        <Command>mplayer \"mymusicfile.mp3\" -vo null -ao pcm -aofile \"menu_audio.pcm\" ; mp2enc -r 48000 -o \"menu_audio.mpa\" &lt; \"menu_audio.pcm\"</Command>\n\
      </Block>\n\
    </Convert>\n\
  </ExecuteInterface>\n\
  <ExecuteInterface>\n\
    <Pal blockname=\"+MENU_NAME+ (PAL +WIDTH+x+HEIGHT+)\">\n\
      <Block name=\"MOVIE_BACKGROUND\">\n\
        <Comment>&lt;p align=\"center\"&gt;This line of command will convert the background movie&lt;BR&gt;into a MPEG2 stream in PAL format +WIDTH+x+HEIGHT+&lt;/p&gt;</Comment>\n\
        <Command default=\"true\">transcode -i -V -J fps 25 --export_asr 2 --keep_asr -Z +WIDTH+x+HEIGHT+ -y jpg,wav -m \"+TEMP_PATH+/audio+MENU_NAME+.wav\" -o background+MENU_NAME+</Command>\n\
      </Block>\n\
      <Block name=\"IMAGE_BACKGROUND\">\n\
        <Comment>&lt;p align=\"center\"&gt;This line of command will convert the background image&lt;BR&gt;into a MPEG2 stream in PAL format +WIDTH+x+HEIGHT+&lt;/p&gt;</Comment>\n\
        <Command>jpegtopnm \"+BACKGROUND_FILE_NAME+\" | ppmtoy4m -n 1 -F25:1 -A59:54 -I t -L | mpeg2enc -f 8 -n p -o \"+TEMP_PATH+/+MENU_NAME+.m2v\"</Command>\n\
        <Command default=\"true\">jpeg2yuv -n 50 -I p -f 25 -j \"+BACKGROUND_FILE_NAME+\" | mpeg2enc -n p -f 8 -o \"+TEMP_PATH+/+MENU_NAME+.m2v\"</Command>\n\
      </Block>\n\
      <Block name=\"EMPTY_SOUND\">\n\
        <Comment>&lt;p align=\"center\"&gt;This line creates an empty sound file to mix into the menu-movie.&lt;/p&gt;</Comment>\n\
        <Command>arecord -f dat -twav -d 1 | mp2enc -r 48000 -o \"+TEMP_PATH+/+MENU_NAME+.mp2\"</Command>\n\
        <Command>arecord -S -M -t 1 -w /dev/stdout | mp2enc -r 48000 -o \"+TEMP_PATH+/+MENU_NAME+.mp2\"</Command>\n\
        <Command>sox -t raw -s -w -c 2 -r 48000 /dev/zero -t wav -c 2 -r 48000 /dev/stdout trim 0 1 | mp2enc -r 48000 -o \"+TEMP_PATH+/+MENU_NAME+.mp2\"</Command>\n\
        <Command default=\"true\">dd if=/dev/zero bs=4 count=1920 | toolame -b128 -s 48 /dev/stdin \"+TEMP_PATH+/+MENU_NAME+.mp2\"</Command>\n\
      </Block>\n\
      <Block name=\"MPLEX\">\n\
        <Comment>&lt;p align=\"center\"&gt;The following command will multiplex the sound file into the menu-movie.&lt;/p&gt;</Comment>\n\
        <Command default=\"true\">mplex -f 8 -o \"+TEMP_PATH+/+MENU_NAME+.mpg\" \"+TEMP_PATH+/+MENU_NAME+.m2v\" +SOUND_LIST+</Command>\n\
      </Block>\n\
      <Block name=\"SPUMUX\">\n\
        <Comment>&lt;p align=\"center\"&gt;Here we use spumux to add the subpicture (buttons) the the mpeg2 video.&lt;/p&gt;</Comment>\n\
        <Command default=\"true\">spumux \"+TEMP_PATH+/+MENU_NAME+.xml\" &lt; \"+TEMP_PATH+/+MENU_NAME+.mpg\" &gt; \"+TEMP_PATH+/+MENU_NAME+_menu.mpg\"</Command>\n\
      </Block>\n\
    </Pal>\n\
    <Ntsc blockname=\"+MENU_NAME+ (NTSC +WIDTH+x+HEIGHT+)\">\n\
      <Block name=\"MOVIE_BACKGROUND\">\n\
        <Comment>&lt;p align=\"center\"&gt;This line of command will convert the background movie&lt;BR&gt;into a MPEG2 stream in NTSC format +WIDTH+x+HEIGHT+&lt;/p&gt;</Comment>\n\
        <Command default=\"true\">transcode -i -V -J fps 29.970 --export_asr 2 --keep_asr -Z +WIDTH+x+HEIGHT+ -y jpg,wav -m \"+TEMP_PATH+/audio+MENU_NAME+.wav\" -o background+MENU_NAME+</Command>\n\
      </Block>\n\
      <Block name=\"IMAGE_BACKGROUND\">\n\
        <Comment>&lt;p align=\"center\"&gt;This line of command will convert the background image&lt;BR&gt;into a MPEG2 stream in NTSC format +WIDTH+x+HEIGHT+&lt;/p&gt;</Comment>\n\
        <Command>jpegtopnm \"+BACKGROUND_FILE_NAME+\" | ppmtoy4m -n 1 -F30000:1001 -A10:11 -I t -L | mpeg2enc -f 8 -n n -o \"+TEMP_PATH+/+MENU_NAME+.m2v\"</Command>\n\
        <Command default=\"true\">jpeg2yuv -n 50 -I p -f 29.97 -j \"+BACKGROUND_FILE_NAME+\" | mpeg2enc -n n -f 8 -o \"+TEMP_PATH+/+MENU_NAME+.m2v\"</Command>\n\
      </Block>\n\
      <Block name=\"EMPTY_SOUND\">\n\
        <Comment>&lt;p align=\"center\"&gt;This line creates an empty sound file to mix into the menu-movie.&lt;/p&gt;</Comment>\n\
        <Command default=\"true\">arecord -f dat -twav -d 1 /dev/stdout | mp2enc -r 48000 -o \"+TEMP_PATH+/+MENU_NAME+.mp2\"</Command>\n\
        <Command>arecord -S -M -t 1 -w /dev/stdout | mp2enc -r 48000 -o \"+TEMP_PATH+/+MENU_NAME+.mp2\"</Command>\n\
        <Command>sox -t raw -s -w -c 2 -r 48000 /dev/zero -t wav -c 2 -r 48000 /dev/stdout trim 0 1 | mp2enc -r 48000 -o \"+TEMP_PATH+/+MENU_NAME+.mp2\"</Command>\n\
        <Command>dd if=/dev/zero bs=4 count=1601.6 | toolame -b128 -s 48 /dev/stdin \"+TEMP_PATH+/+MENU_NAME+.mp2\"</Command>\n\
      </Block>\n\
      <Block name=\"MPLEX\">\n\
        <Comment>&lt;p align=\"center\"&gt;The following command will multiplex the sound file into the menu-movie.&lt;/p&gt;</Comment>\n\
        <Command default=\"true\">mplex -f 8 -o \"+TEMP_PATH+/+MENU_NAME+.mpg\" \"+TEMP_PATH+/+MENU_NAME+.m2v\" +SOUND_LIST+</Command>\n\
      </Block>\n\
      <Block name=\"SPUMUX\">\n\
        <Comment>&lt;p align=\"center\"&gt;Here we use spumux to add the subpicture (buttons) the the mpeg2 video.&lt;/p&gt;</Comment>\n\
        <Command default=\"true\">spumux \"+TEMP_PATH+/+MENU_NAME+.xml\" &lt; \"+TEMP_PATH+/+MENU_NAME+.mpg\" &gt; \"+TEMP_PATH+/+MENU_NAME+_menu.mpg\"</Command>\n\
      </Block>\n\
    </Ntsc>\n\
  </ExecuteInterface>\n\
  <ExecuteInterface>\n\
    <Dvdauthor blockname=\"dvdauthor\" color=\"#F92F5E\">\n\
      <Block>\n\
        <Comment>Here we start dvdauthor with the generated xml file.</Comment>\n\
        <Command>dvdauthor -x \"+DVDAUTHOR_XML+\"</Command>\n\
      </Block>\n\
    </Dvdauthor>\n\
  </ExecuteInterface>\n\
  <ExecuteInterface hidden=\"true\">\n\
    <Burning blockname=\"Burn DVD\" color=\"#E0EE8F\">\n\
      <Block>\n\
        <Comment>Last we burn the DVD.</Comment>\n\
        <Command>growisofs -Z /dev/dvd -dvd-video +PROJECT_PATH+/</Command>\n\
        <Comment>This command will format a empty -RW DVD (only needed for older drives).</Comment>\n\
        <Command>dvd+rw-format -f /dev/srcd0</Command>\n\
        <Comment>This command generates an iso image of the DVD.</Comment>\n\
        <Command>mkisofs -dvd-video -udf -o +TEMP_PATH+/dvd.iso +PROJECT_PATH+/</Command>\n\
        <Comment>This command burns the iso DVD image to DVD</Comment>\n\
        <Command>dvdrecord -dao speed=2 dev=0,0,0 +TEMP_PATH+/dvd.iso</Command>\n\
      </Block>\n\
    </Burning>\n\
  </ExecuteInterface>\n\
</QDVDAuthorInitFile>\n").arg((float)INIT_VERSION);
  		stream << (const char *)qsResetInitFile << "\n";
		initFile.close();
	}
}

*/

